//  This file was generated by LevelHelper
//  http://www.levelhelper.org
//
//  LevelHelperLoader.mm
//  Created by Bogdan Vladu
//  Copyright 2011 Bogdan Vladu. All rights reserved.
////////////////////////////////////////////////////////////////////////////////
//
//  This software is provided 'as-is', without any express or implied
//  warranty.  In no event will the authors be held liable for any damages
//  arising from the use of this software.
//  Permission is granted to anyone to use this software for any purpose,
//  including commercial applications, and to alter it and redistribute it
//  freely, subject to the following restrictions:
//  The origin of this software must not be misrepresented; you must not
//  claim that you wrote the original software. If you use this software
//  in a product, an acknowledgment in the product documentation would be
//  appreciated but is not required.
//  Altered source versions must be plainly marked as such, and must not be
//  misrepresented as being the original software.
//  This notice may not be removed or altered from any source distribution.
//  By "software" the author refers to this code file and not the application 
//  that was used to generate this file.
//
////////////////////////////////////////////////////////////////////////////////

#import "LHSettings.h"
#import "cocos2d.h"
#import "LevelHelperLoader.h"

#import "LHLayer.h"
#import "LHSprite.h"
#import "LHBezier.h"
#import "LHJoint.h"

@implementation LHSettings
////////////////////////////////////////////////////////////////////////////////

@synthesize useRetinaOnIpad;
@synthesize lhPtmRatio;
@synthesize customAlpha;
@synthesize convertLevel;
@synthesize levelPaused;
@synthesize isCoronaUser;
@synthesize preloadBatchNodes;
@synthesize device;
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
+ (LHSettings*)sharedInstance{
	static id sharedInstance = nil;
	if (sharedInstance == nil){
		sharedInstance = [[LHSettings alloc] init];
	}
    return sharedInstance;
}
////////////////////////////////////////////////////////////////////////////////
-(void)dealloc
{
#ifndef LH_ARC_ENABLED
    [hdSuffix release];
    [hd2xSuffix release];
    
    [markedSprites release];
    [markedBeziers release];
    [markedJoints release];
    [allLHMainLayers release];
	[super dealloc];
#endif
}
////////////////////////////////////////////////////////////////////////////////
- (id)init
{
	self = [super init];
	if (self != nil) {
        
        markedSprites = [[NSMutableSet alloc] init];
        markedBeziers = [[NSMutableSet alloc] init];
        markedJoints  = [[NSMutableSet alloc] init];
        allLHMainLayers = [[NSMutableArray alloc] init];
        
		useRetinaOnIpad = true;
		convertLevel = true;
		lhPtmRatio = 32.0f;
		customAlpha = 1.0f;
		convertRatio = CGPointMake(1, 1);
        realConvertRatio = CGPointMake(1, 1);
		newBodyId = 0;
        stretchArt = true;
        possitionOffset = CGPointMake(0.0f, 0.0f);
        levelPaused = false;
		//imagesFolder = [[NSMutableString alloc] init];
        isCoronaUser = false;
        preloadBatchNodes = false;
        
        activeBox2dWorld = NULL;
        
        hdSuffix = [[NSMutableString alloc] initWithString:@"-hd"];
        hd2xSuffix = [[NSMutableString alloc] initWithString:@"-ipadhd"];
	}
	return self;
}
////////////////////////////////////////////////////////////////////////////////

-(void)addLHMainLayer:(LHLayer*)layer{
    [allLHMainLayers addObject:layer];
}
-(void)removeLHMainLayer:(LHLayer*)layer{
    [allLHMainLayers removeObject:layer];
}
-(NSArray*)allLHMainLayers{
    return allLHMainLayers;
}

-(b2World*)activeBox2dWorld{
    return activeBox2dWorld;
}
-(void)setActiveBox2dWorld:(b2World*)world{
    activeBox2dWorld = world;
}

-(void)setHDSuffix:(NSString*)suffix{
    if(nil == suffix) {[hdSuffix setString:@"-hd"]; return;}
    [hdSuffix setString:suffix];
}
-(NSString*)hdSuffix{
    return hdSuffix;
}

-(void)setHD2xSuffix:(NSString*)suffix{
    if(nil == suffix) {[hd2xSuffix setString:@"-ipadhd"]; return;}
    [hd2xSuffix setString:suffix];
    
}
-(NSString*)hd2xSuffix{
    return hd2xSuffix;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
-(void) markSpriteForRemoval:(LHSprite*)ccsprite{
    if(nil == ccsprite) return;
    [markedSprites addObject:ccsprite];
}
//------------------------------------------------------------------------------
-(void) markBezierForRemoval:(LHBezier*) node{
    if(nil == node) return;
    [markedBeziers addObject:node];    
}
//------------------------------------------------------------------------------
-(void) markJointForRemoval:(LHJoint*)jt{
    if(jt != NULL) [markedJoints addObject:jt];
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
-(void) removeMarkedSprites{
    for(LHSprite* spr in markedSprites){
        [spr removeSelf];
    }
    [markedSprites removeAllObjects];
}
//------------------------------------------------------------------------------
-(void) removeMarkedBeziers{
    for(LHBezier* node in markedBeziers){
        [node removeSelf];
    }
    [markedBeziers removeAllObjects];   
}
//------------------------------------------------------------------------------
-(void) removeMarkedJoints{
    for(LHJoint* jt in markedJoints){
        [jt removeSelf];
    }
    [markedJoints removeAllObjects];
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------


-(int)newBodyId{
	return newBodyId++;
}

-(NSString*)imagePath:(NSString*)file 
{
    if([self isIpad])
    {   
        if(device != 1)//if ipad only then we dont need to apply transformations
        {
            if(CC_CONTENT_SCALE_FACTOR() == 2)
            {
                //we have ipad retina
                file = [NSString stringWithFormat:@"%@%@.%@", [file stringByDeletingPathExtension], hd2xSuffix, [file pathExtension]];
            }
            else {
                //we have normal ipad - lets use the HD image
                file = [NSString stringWithFormat:@"%@%@.%@", [file stringByDeletingPathExtension], hdSuffix, [file pathExtension]];
            }
        }
        
        NSString *fullpath = [CCFileUtils fullPathFromRelativePath:file];
                
        if([[NSFileManager defaultManager] fileExistsAtPath:fullpath]){
            return fullpath;
        }
    }
    return file;
}

-(CGPoint)transformedPointToCocos2d:(CGPoint)point{
    
    CGSize winSize = [[CCDirector sharedDirector] winSize];
    
    point.x *= [self convertRatio].x;
    point.y *= [self convertRatio].y;
    point.y = winSize.height - point.y;
    CGPoint pos_offset = [self possitionOffset];
    point.x += pos_offset.x;
    point.y -= pos_offset.y;
    
    return point;
}
-(CGPoint)transformedPoint:(CGPoint)point forImage:(NSString*)image{
        
    if(device != 0 && device != 1) //iphone //ipad
    {
        point.x *= [self convertRatio].x;
        point.y *= [self convertRatio].y;

        if ([image rangeOfString:hdSuffix].location != NSNotFound ||
            [image rangeOfString:hd2xSuffix].location != NSNotFound) {
            point.x /=2.0f;
            point.y /=2.0f;        
        }
    }
    return point;    
}

-(CGRect)transformedTextureRect:(CGRect)rect forImage:(NSString*)image
{
    if(device != 0 && device != 1) //iphone //ipad
    {
        if ([image rangeOfString:hdSuffix].location != NSNotFound ||
            [image rangeOfString:hd2xSuffix].location != NSNotFound) {
        
            rect = CGRectMake(rect.origin.x*2.0f, rect.origin.y*2.0f, 
                              rect.size.width*2.0f, rect.size.height*2.0f);
        }
    }

    return rect;    
}

-(bool)isHDImage:(NSString*)image
{
    if(device == 0 || device == 1) //iphone //ipad
        return false;
    
    if ([image rangeOfString:hdSuffix].location == NSNotFound&&
        [image rangeOfString:hd2xSuffix].location == NSNotFound) {
        return false;
    }
    return true;
}

-(bool)isIpad
{
    if(![self useRetinaOnIpad]){
        return false;
    }

    
#ifndef __MAC_OS_X_VERSION_MAX_ALLOWED
	
    
#ifdef __IPHONE_OS_VERSION_MAX_ALLOWED
#if __IPHONE_3_2 <= __IPHONE_OS_VERSION_MAX_ALLOWED
	UIDevice* thisDevice = [UIDevice currentDevice];
	if ([thisDevice respondsToSelector:@selector(userInterfaceIdiom)]){
		if(thisDevice.userInterfaceIdiom == UIUserInterfaceIdiomPad){
			return true;
		}
	}
	else{
		return false;
	}
#else
	return false;
#endif
    
#endif
    
#else
	return true;
#endif
	
	return false;
}

-(void) setStretchArt:(bool)value{
    stretchArt = value;
    possitionOffset.x =0.0f;
    possitionOffset.y =0.0f;   
}

-(void) setConvertRatio:(CGPoint)val// usesCustomSize:(bool)usesCustomSize{
{
	convertRatio = val;
    realConvertRatio = val;
    if(!stretchArt)
    {
        if([self isIpad])
        {
            if(convertRatio.x > 1.0 || convertRatio.y > 1.0f)
            {
                convertRatio.x = 2.0f;
                convertRatio.y = 2.0f;
                
#ifdef LH_SCENE_TESTER
    convertRatio.x = 1.0f;
    convertRatio.y = 1.0f;
#endif
                
                if([[CCDirector sharedDirector] winSize].width == 1024.0f)
                {
                    possitionOffset.x = 32.0f;
                    possitionOffset.y = 64.0f;   
                }
                else {
                    possitionOffset.x = 64.0f;
                    possitionOffset.y = 32.0f;
                }
                
#ifdef LH_SCENE_TESTER
                if([[CCDirector sharedDirector] winSize].width == 512.0f)
                {
                    possitionOffset.x = 16.0f;
                    possitionOffset.y = 32.0f;  
                }
                else {
                    possitionOffset.x = 32.0f;
                    possitionOffset.y = 16.0f;
                }
#endif

            }
            
            if(device == 3)
            {
                convertRatio.x = 1.0f;
                convertRatio.y = 1.0f;
            }
        }
    }
}
-(CGPoint) possitionOffset{
    return possitionOffset;
}

-(CGPoint) convertRatio{
	
	if(!convertLevel)
		return CGPointMake(1, 1);
	
	return convertRatio;
}


-(CGPoint) realConvertRatio{
    if(!convertLevel)
		return CGPointMake(1, 1);
	
	return realConvertRatio;
}


@end
